/******************************************************************************
 * C++-Programm 'example.cpp' im Verzeichnis '~/mycpp/jmb/'                   *
 *                                                                            *
 * Funktion:    "Zerlegung einer Zahl in ihre Primfaktoren"                   *
 *                                                                            *
 * J.M.B.   [URL: https://www.jmb-edu.de/cpp_programming.html#progcppexamp]   *
 *                                                                            *
 * Lizenz:  (c) 2019  unter GPLv3: https://www.gnu.org/licenses/gpl-3.0.txt   *
 *                                                                            *
 * Erstellung:           05.08.2019                                           *
 *                                                                            *
 * Letzte Umgestaltung:  04.09.2019  (Phase 1; vgl. Phase 2 vom 03.09.2019)   *
 ******************************************************************************
 */

# include <iostream>                      // fuer std::cin, std::cout, std::endl
# include <string>                        // fuer std::stoi
# include <math.h>                        // fuer sqrt

/* Gebrauch von ANSI-Farben in kompatiblen Terminals:
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Effekt          Code   |  Farbe   Vordergrund Hintergrund |  ASCII 27 = \033
 * zurücksetzen      0    |  schwarz       30         40     |   = ESC-Zeichen
 * fett/helle VG-Fa. 1    |  rot           31         41     |
 * unterstrichen     4    |  gruen         32         42     | Z.B. hell rot
 * invertiert        7    |  gelb/orange   33         43     |      auf schwarz
 * fett/hell aus    21    |  blau          34         44     |   "\033[4;31;40m"
 * unterstri. aus   24    |  magenta       35         45     |
 * invertiert aus   27    |  cyan          36         46     | Reset: "\033[0m"
 *   * Just play ! *      |  weiss/grau    37         47     |  * Have fun! *
 */
#define RESET             "\033[0m"          /* Reset to Defaul: zuruecksetzen*/
#define ULLWHITEONYELLOW  "\033[1;4;37;43m"  /* unterstrichen weiss auf orange*/
#define ULLYELLOWONGREEN  "\033[1;4;33;42m"  /* unterstrichen gelb auf gruen  */
#define ULLYELLOWONYELLOW "\033[1;4;33;43m"  /* unterstrichen gelb auf orange */
#define ULLREDONBLACK     "\033[1;4;31;40m"  /* unterstrichen rosa auf schwarz*/

void primeFactors(int n)               // Eigene Funktion zur PF-Ausgabe
{
  using std::cout;                     // cout bedeutet nun std::cout
  using std::endl;                     // endl bedeutet nun std::endl
  cout << "Primzahlfaktoren von " << n << " sind:\n";
  /* 1. Schritt: */
  // Anzahl 2-en ausgeben, durch die n (vor 1. Schritt) dividiert werden kann
  while (n%2 == 0)                     // wenn bei Division kein Rest
  {
    cout << "2, ";                     // jeder Faktor wird ausgegeben
    n = n/2;
  }
  /* 2. Schritt: */
  // n (Anfang 2. Schritt) muss nun ungerade sein - Primfaktoren 2 wurden
  // entdeckt, somit kann man immer eine Zahl uebergehen, daher i=i+2
  for (int i = 3; i <= sqrt(n); i = i+2)
  {
    // Solange n durch i teilbar ist, wird i ausgegeben (analog 2 oben)
    while (n%i == 0)                   // wenn bei Division kein Rest
    {
      cout << i << ", ";               // jeder Faktor wird ausgegeben
      n = n/i;
    }
  }
  /* 3. Schritt: */
  // Wenn for bis i = Wurzel n (n nach 1. Schritt) nicht n=1 (Ende 2. Schritt)
  // liefert, muss n der letzte fehlende Primfaktor der Zerlegung sein
  if (n > 2)
  {
    cout << n << ".";                  // Ausgabe n, da letzter fehlender PF
  }
  cout << endl;
}

int main(int argc, const char* argv[]) // Hauptfunktion bei Aufruf-Parameter
{ /* Begruessung */
  std::cout << ULLYELLOWONGREEN << " * Programm '" << argv[0] << "' zur Ausgabe der Primfaktoren *" << RESET << "\n";
  /* Zahl ermitteln, die in PFs aufgespalten werden soll */
  int wert = 0;
  if (argc > 1)                        // Wurde neben Aufrufnamen ein Parameter
  {                                    //   angegeben?
    wert = std::stoi(argv[1]);         // Dann sollte dies die Zahl sein.
    if (wert < 2)                      // Zahlen kleiner 2 machen keinen Sinn!
    {
      std::cout << ULLREDONBLACK << "Fehler:" << RESET << " Der Aufrufparameter war kleiner als 2!\n";
    }
  }
  while (wert < 2)                     // Bis wert mindestens 2 ist einlesen
  {
    std::cout << "Gib eine ganze Zahl > 1 ein: ";
    std::cin  >> wert;                 // Eingabewert in Variable wert lesen
    if(!std::cin)                      // Lese-Status pruefen ...
    {
      std::cout << ULLREDONBLACK << "Fehler:" << RESET << " Unerlaubte Eingabe!\n";
      return 1;                        // Abbruch mit Fehlercode 1
    }
  }
  /* PFs ermitteln und mit "," getrennt ausgeben */
  primeFactors(wert);                  // Funktionsaufruf fuer PFs
  /* Verabschiedung */
  std::cout << ULLWHITEONYELLOW << " * That's all, folks! " << ULLYELLOWONYELLOW << ";)" << ULLWHITEONYELLOW << " * -<(c) J.M.B. 2019, GPLv3>~-" << RESET << "\n";
  return 0;            // Uebergabewert (= Fehlercode) an Betriebssystem/Shell
}
/******************************************************************************
 * Das war's - ein erstes kleines Programm - nicht zu trivial ...    :))      *
 ******************************************************************************
 */
